(in-package :cl-user)

(defpackage :cl-prevalence-bug
  (:use :cl :cl-prevalence))

(in-package :cl-prevalence-bug)

(defvar *test-tmp-dir* #p"/tmp/test-cl-prevalence-bug/")

(defvar *test-system* nil)

(defclass person (object-with-id)
  ((firstname :initarg :firstname :initform "" :accessor firstname)
   (lastname :initarg :lastname :initform "" :accessor lastname)
   (parent :initarg :parent :initform nil :accessor parent)))

(defun make-person (&rest slots)
  (let ((slots-and-values
	 (loop for (slot value) on slots by #'cddr
	    collect (list (intern (string-upcase slot)
				  'cl-prevalence-bug)
			  value))))
    (execute-transaction
     (tx-create-object *test-system* 'person
		       slots-and-values))))

(defun setup ()
 ;; create new prevalence system
  (let ((directory *test-tmp-dir*))
    (when (probe-file directory)
      (dolist (pathname (directory (merge-pathnames "*.xml" directory)))
	(delete-file pathname)))
    (ensure-directories-exist directory)
    (setf *test-system* (make-prevalence-system directory))
    (execute-transaction (tx-create-id-counter *test-system*))
    (assert (zerop (get-root-object *test-system* :id-counter)))))

(defvar *ma-id*)
(defvar *joe-id*)

(defun make-joe-and-ma ()
  (let* ((ma (make-person :firstname "Ma" :lastname "Dalton"))
	 (joe (make-person :firstname "Joe" :lastname "Dalton"
					    :parent ma)))
    (setf *ma-id* (slot-value ma 'id))
    (setf *joe-id* (slot-value joe 'id))))

(defun check-joe-and-ma ()
  ;; check that the parent of JOE is MA
  (assert
   (eq (find-object-with-id *test-system* 'person *ma-id*)
       (parent (find-object-with-id *test-system* 'person *joe-id*)))))

(defun destroy-and-restart ()
  ;; throw the prevalence instance and start over
  (cl-prevalence::close-open-streams *test-system*)
  (setf *test-system* (make-prevalence-system *test-tmp-dir*)))

(defun snapshot-destroy-and-restart ()
  (snapshot *test-system*)
  (destroy-and-restart))


#|

(progn
 (setup)
 (make-joe-and-ma)
 (check-joe-and-ma)			; ok
 (snapshot-destroy-and-restart)
 (check-joe-and-ma))			; ok

(progn
 (setup)
 (make-joe-and-ma)
 (check-joe-and-ma)			; ok
 (destroy-and-restart)
 (check-joe-and-ma))			; assertion fails

|#
